package log

import org.apache.spark.sql.Row
import org.apache.spark.sql.types.{LongType, StringType, StructField, StructType}

/**
  * 访问日志转换（输入==》输出）工具类
  *
  */
object AccessConvertUtil {
  /**
    * 定义输出数据类型的字段
    */
  val struct = StructType(
    Array(
      StructField("url",StringType),
      StructField("cmsType",StringType),
      StructField("cmsId",LongType),
      StructField("traffic",LongType),
      StructField("ip",StringType),
      StructField("city",StringType),
      StructField("time",StringType),
      StructField("day",StringType)
    )
  )
  /**
    * 是根据输入的每一行信息转换成输出的 样式
      @param log 输入的每一行记录信息
               2017-05-11 14:09:14	http://www.imooc.com/video/4500	304	218.75.35.226
    */
  def parseLog(log:String)={
    try {
      val splits = log.split("\t")
      val url = splits(1)
      val traffic = splits(2).toLong
      val ip = splits(3)
      val domain = "http://www.imooc.com/"
      val cms = url.substring(url.indexOf(domain)+domain.length)
      var cmsTypedId = cms.split("/")
      var cmsType = ""
      var cmsId = 0l

      if (cmsTypedId.length>1){
        cmsType = cmsTypedId(0)
        if (cmsId.isInstanceOf[Long]){
          cmsId = cmsTypedId(1).toLong
        }else{
          cmsId = 0l
        }
      }
      val city = IpUtils.getCity(ip)
      val time  = splits(0)
      val day = time.substring(0,10).replaceAll("-","")

      Row(url,cmsType,cmsId,traffic,ip,city,time,day)
    }catch {
      case e: Exception=>Row("","",0l,0l,"","","","")
    }
  }
}
